@@ -41,9 +41,10 @@ except ImportError:
     from urllib import quote
 
 
+import py_eureka_client.http_client as http_client
 from py_eureka_client.logger import get_logger
 from py_eureka_client.__dns_txt_resolver import get_txt_dns_record
-import py_eureka_client.http_client as http_client
+from py_eureka_client.__aws_info_loader import AmazonInfo
 
 
 try:
@@ -101,10 +102,12 @@ _RENEWAL_INTERVAL_IN_SECS = 30
 _DURATION_IN_SECS = 90
 _DEFAULT_DATA_CENTER_INFO = "MyOwn"
 _DEFAULT_DATA_CENTER_INFO_CLASS = "com.netflix.appinfo.InstanceInfo$DefaultDataCenterInfo"
+_AMAZON_DATA_CENTER_INFO_CLASS = "com.netflix.appinfo.AmazonInfo"
 """
-Default encoding
+Default configurations
 """
 _DEFAULT_ENCODING = "utf-8"
+_DEFAUTL_ZONE = "default"
 
 ### =========================> Base Mehods <======================================== ###
 ### Beans ###
@@ -139,12 +142,13 @@ class Applications:
             self.__applications.append(application)
             self.__application_name_dic[application.name] = application
 
-    def get_application(self, app_name):
+    def get_application(self, app_name=""):
         with self.__app_lock:
+            aname = app_name.upper()
             if app_name in self.__application_name_dic:
-                return self.__application_name_dic[app_name]
+                return self.__application_name_dic[aname]
             else:
-                return Application(name=app_name)
+                return Application(name=aname)
 
 
 class Application:
@@ -192,12 +196,12 @@ class Application:
 
     def up_instances_in_zone(self, zone):
         with self.__inst_lock:
-            _zone = zone if zone else "defaultZone"
+            _zone = zone if zone else _DEFAUTL_ZONE
             return [item for item in self.__instances_dict.values() if item.status == INSTANCE_STATUS_UP and item.zone == _zone]
 
     def up_instances_not_in_zone(self, zone):
         with self.__inst_lock:
-            _zone = zone if zone else "defaultZone"
+            _zone = zone if zone else _DEFAUTL_ZONE
             return [item for item in self.__instances_dict.values() if item.status == INSTANCE_STATUS_UP and item.zone != _zone]
 
 
@@ -224,9 +228,11 @@ class DataCenterInfo:
 
     def __init__(self,
                  name=_DEFAULT_DATA_CENTER_INFO,  # Netflix, Amazon, MyOwn
-                 className=_DEFAULT_DATA_CENTER_INFO_CLASS):
+                 className=_DEFAULT_DATA_CENTER_INFO_CLASS,
+                 metadata={}):
         self.name = name
         self.className = className
+        self.metadata = metadata if metadata else {}
 
 
 class PortWrapper:
@@ -291,10 +297,13 @@ class Instance(object):
 
     @property
     def zone(self):
+        if self.dataCenterInfo and self.dataCenterInfo.name == "Amazon" \
+                and self.dataCenterInfo.metadata and "availability-zone" in self.dataCenterInfo.metadata:
+            return self.dataCenterInfo.metadata["availability-zone"]
         if self.metadata and "zone" in self.metadata and self.metadata["zone"]:
             return self.metadata["zone"]
         else:
-            return "defaultZone"
+            return _DEFAUTL_ZONE
 
 
 ########################## Basic functions #################################
@@ -339,6 +348,8 @@ def register(eureka_server, instance):
         'lastDirtyTimestamp': str(instance.lastDirtyTimestamp),
         'isCoordinatingDiscoveryServer': str(instance.isCoordinatingDiscoveryServer).lower()
     }
+    if instance.dataCenterInfo.metadata:
+        instance_dic["dataCenterInfo"]["metadata"] = instance.dataCenterInfo.metadata
     _register(eureka_server, instance_dic)
 
 
@@ -459,7 +470,7 @@ def _build_instance(xml_node):
         elif child_node.tag == "countryId":
             instance.countryId = int(child_node.text)
         elif child_node.tag == "dataCenterInfo":
-            instance.dataCenterInfo = DataCenterInfo(name=child_node.text, className=child_node.attrib["class"])
+            instance.dataCenterInfo = _build_data_center_info(child_node)
         elif child_node.tag == "hostName":
             instance.hostName = child_node.text
         elif child_node.tag == "status":
@@ -484,6 +495,19 @@ def _build_instance(xml_node):
     return instance
 
 
+def _build_data_center_info(xml_node):
+    class_name = xml_node.attrib["class"]
+    name = ""
+    metadata = {}
+    for child_node in xml_node:
+        if child_node.tag == "name":
+            name = child_node.text
+        elif child_node.tag == "metadata":
+            metadata = _build_metadata(child_node)
+
+    return DataCenterInfo(name=name, className=class_name, metadata=metadata)
+
+
 def _build_metadata(xml_node):
     metadata = {}
     for child_node in list(xml_node):
@@ -571,7 +595,9 @@ class EurekaServerConf(object):
                  zone=""):
         self.__servers = {}
         self.region = region
-        self.zone = _zone = zone if zone else "defaultZone"
+        self.__zone = zone
+        self.__eureka_availability_zones = eureka_availability_zones
+        _zone = zone if zone else _DEFAUTL_ZONE
         if eureka_domain:
             zone_urls = get_txt_dns_record("txt.%s.%s" % (region, eureka_domain))
             for zone_url in zone_urls:
@@ -594,7 +620,17 @@ class EurekaServerConf(object):
         if _zone in self.__servers_not_in_zone:
             del self.__servers_not_in_zone[_zone]
 
-    def _format_url(self, server_url="", eureka_protocol="http",
+    @property
+    def zone(self):
+        if self.__zone:
+            return self.__zone
+        elif self.__eureka_availability_zones:
+            return self.__eureka_availability_zones.keys()[0]
+        else:
+            return _DEFAUTL_ZONE
+
+    def _format_url(self, server_url="",
+                    eureka_protocol="http",
                     eureka_basic_auth_user="",
                     eureka_basic_auth_password="",
                     eureka_context="eureka/v2"):
@@ -669,7 +705,7 @@ class EurekaClient(object):
                 "us-east-1d": "http://ec2-552-627-568-170.compute-1.amazonaws.com:7001/eureka/v2/",
                 "us-east-1e": "http://ec2-500-179-285-592.compute-1.amazonaws.com:7001/eureka/v2/"}, 
                 zone="us-east-1c",
-                app_name="python_module_1", 
+                app_name="python_module_1",
                 instance_port=9090,
                 data_center_name="Amazon")
 
@@ -748,7 +784,7 @@ class EurekaClient(object):
 
     * region: The region when using DNS discovery.
 
-    * zone: Which zone your instances belong to, default is `defaultZone`.
+    * zone: Which zone your instances belong to, default is `default`.
 
     * eureka_availability_zones: The zones' url configurations.
 
@@ -897,7 +933,7 @@ class EurekaClient(object):
         mdata = {
             'management.port': str(instance_port)
         }
-        if zone and zone != "defaultZone":
+        if zone:
             mdata["zone"] = zone
         mdata.update(metadata)
         self.__instance = {
@@ -915,7 +951,7 @@ class EurekaClient(object):
             },
             'countryId': 1,
             'dataCenterInfo': {
-                '@class': _DEFAULT_DATA_CENTER_INFO_CLASS,
+                '@class': _AMAZON_DATA_CENTER_INFO_CLASS if data_center_name == "Amazon" else _DEFAULT_DATA_CENTER_INFO_CLASS,
                 'name': data_center_name
             },
             'leaseInfo': {
@@ -935,6 +971,8 @@ class EurekaClient(object):
             'secureVipAddress': secure_vip_addr if secure_vip_addr != "" else app_name.lower(),
             'isCoordinatingDiscoveryServer': str(is_coordinating_discovery_server).lower()
         }
+        if data_center_name == "Amazon":
+            self.__instance["dataCenterInfo"]["metadata"] = self.__load_ec2_metadata_dict()
 
         # For discovery
         self.__remote_regions = remote_regions if remote_regions is not None else []
@@ -945,6 +983,32 @@ class EurekaClient(object):
 
         self.__application_mth_lock = RLock()
 
+    def __load_ec2_metadata_dict(self):
+        # instance metadata
+        amazon_info = AmazonInfo()
+        mac = amazon_info.get_ec2_metadata('mac')
+        if mac:
+            vpc_id = amazon_info.get_ec2_metadata('network/interfaces/macs/%s/vpc-id' % mac)
+        else:
+            vpc_id = ""
+        metadata = {
+            'instance-id': amazon_info.get_ec2_metadata('instance-id'),
+            'ami-id': amazon_info.get_ec2_metadata('ami-id'),
+            'instance-type': amazon_info.get_ec2_metadata('instance-type'),
+            'local-ipv4': amazon_info.get_ec2_metadata('local-ipv4'),
+            'local-hostname': amazon_info.get_ec2_metadata('local-hostname'),
+            'availability-zone': amazon_info.get_ec2_metadata('placement/availability-zone'),
+            'public-hostname': amazon_info.get_ec2_metadata('public-hostname'),
+            'public-ipv4': amazon_info.get_ec2_metadata('public-ipv4'),
+            'mac': mac,
+            'vpcId': vpc_id
+        }
+        # accountId
+        doc = amazon_info.get_instance_identity_document()
+        if doc and "accountId" in doc:
+            metadata["accountId"] = doc["accountId"]
+        return metadata
+
     @property
     def should_register(self):
         return self.__should_register
@@ -1001,10 +1065,10 @@ class EurekaClient(object):
         else:
             self.__try_eureka_server_regardless_zones(fun)
 
-    def __try_eureka_servers_in_list(self, fun, eureka_servers=[], zone="defaultZone"):
+    def __try_eureka_servers_in_list(self, fun, eureka_servers=[], zone=_DEFAUTL_ZONE):
         with self.__net_lock:
             ok = False
-            _zone = zone if zone else "defaultZone"
+            _zone = zone if zone else _DEFAUTL_ZONE
             for url in eureka_servers:
                 url = url.strip()
                 try:
